# Lab 2

## Assignment 1

1. Underlying model:

   $$\mathbf{y}=\mathbf{X \theta + \epsilon}$$

   Very bad prediction, much overfitting. Too many features per obsevations.

   - $\text{MSE}_{\text{train}} = 0.00570911701403287$
   - $\text{MSE}_{\text{test}} = 722.429419254052$

2. The cost function to be optimized:

   $$\frac{1}{N}\sum_{i=1}^{n}(y_i - \theta_0-\theta_1x_{1i}-...-\theta_px_{pi})^2 + \lambda \sum_{j=1}^{p}|\theta_j|$$

3. For three features $\lambda \in \{0.8530452, 0.7772630, 0.7082131\}$

   ![assignment1-3](./figures/assignment1-3.png)

   In the plot less coefficients are not zero with higher $\lambda$.

4. Coefficients go towards zero att infinity, i.e. not possible to only have three features.

   ![assignment1-4](./figures/assignment1-4.png)

5. Optimal $\lambda = 0.05744535$ with $8$ variables.

   ![assignment1-5-1](./figures/assignment1-5-1.png)

   CV-score gets smaller with smaller lambda.

   Optimal $\lambda$ is not statistically significantly better than $\lambda = \log(-4)$, looking at the MSE. However it does double the amount of coefficients that are not zero.

   ![assignment1-5-2](./figures/assignment1-5-2.png)

   Predictions are decent since we can see a clear trend.

## Assignment 2

2. Models:

   - _Decision Tree with default settings_

     - $\text{MSE}_\text{train} = 0.1048$
     - $\text{MSE}_\text{valid} = 0.1092679$
     - No. terminal nodes: $6$

   - _Decision Tree with smallest allowed node size equal to 7000_

     - $\text{MSE}_\text{train} = 0.1048$
     - $\text{MSE}_\text{valid} = 0.1092679$
     - No. terminal nodes: $5$

   - _Decision trees minimum deviance to 0.0005_

     - $\text{MSE}_\text{train} = 0.09362$
     - $\text{MSE}_\text{valid} = 0.1118484$
     - No. terminal nodes: $122$

   It is evident that the tree with minimum deviance to 0.0005 is the best model due to having the lowest misclassification rate for validation data.

   Changing the deviance to lower will yield a larger tree due to the stopping condition being more strict, craving more purity for the nodes.

   Chaning the smallest allowed node size to a higher value will yield a smaller tree since we set are not allowing as complex tree.

   A larger tree with more terminal nodes mean a more complex model.

3. As seen in the graph we get underfitting for trees with sizes up to 50 evident by the fact that we get a higher deviance for training data than validation data. We can still generalize better for validation data which is most likely due to correlation between features and limiting the amount of nodes works as regularization.

   ![assignment2-3](./figures/assignment2-3.png)

   High bias gives underfittning and high variance gives overfitting. We find the best amount of leaves when we balance bias and variance and thus minimising the deviance.

   Optimal number of leaves is found to be $21$. The most important variables seem to be _poutcome_, _month_, _contact_ and _pdays_, looking at the structure of the tree.

4. Confusion matrix:

   |     | no    | yes |
   | --- | ----- | --- |
   | no  | 11812 | 167 |
   | yes | 1294  | 291 |

   Accuracy: $0.8922884$

   F1-score: $0.2848752$

   The model does not have very good predictive power. The accuracy is quite high but it does not take into account the fact that the data is very imbalanced. However F1-score does take it into account and gives a more realistic picture.

5. Confusion matrix:

   |     | no    | yes |
   | --- | ----- | --- |
   | no  | 11822 | 157 |
   | yes | 1309  | 276 |

   With the loss matrix we add an extra penalty for classifying _yes_. This means that we need to be five times as sure of _yes_ than _no_ to classify as _yes_. This is reflected in the confusion matrix as we get a higher rate of _no_-classifications.

   This can be beneficial for assymetric classification problems where it would be more costly to calssify false positive than false negative.

6. ROC curve:

   ![assignment2-6](./figures/assignment2-6.png)

   It is evident that the optimal tree is a little bit better than the logistic regression model.

   A precision-recall curve would probably be a more suitable choice here since the classes are very imbalanced.

## Assignment 3

1. $35$ components are needed to capture $95\%$ of variance. PC1 captures $0.2501699$ and PC2 captures $0.1693597$.

2. There are quite a few features with notable contribution. The five most notable are _medFamInc_, _medIncome_, _PctKids2Par_, _pctWInvInc_ and _PctPopUnderPov_. Many of these seem related in that they relate to money. It is quite logical that less money means more likely to commit crimes.

   ![assignment3-2-1](./figures/assignment3-2-1.png)

   In the PC Scores plot we can see that a high level of PC1 and PC2 results in higher criminal rate.

   ![assignment3-2-2](./figures/assignment3-2-2.png)

3. Training and test error is given by $\text{MSE}_{\text{train}} = 0.06853962$ and $\text{MSE}_{\text{test}} = 0.07749598$. The model is quite decent with low errors.

4. Errors depending on iteration:

   ![assignment3-4](./figures/assignment3-4.png)

   Optimal iteration number according to early stopping criterion (i.e. minimum test error) is $4168$.

   Training and test error for optimal model is given by $\text{MSE}_{\text{train}} = 0.06854035$ and $\text{MSE}_{\text{test}} = 0.07747293$. This is very similar to the results in the previous exercise since we optimise the same cost function.

## Assignment 4

- _What are the practical approaches for reducing the expected new data error, according to the book?_

  In practice the aim is to reduce a small error in production, i.e. $E_{\text{new}}$. Of course in practice we can not calculate $E_{\text{new}}$ but instead one can take the decomposition $E_{\text{new}} = E_{\text{train}} + \text{generalisation gap}$ from which one can draw two conclusions: $E_{\text{new}}$ will on averag be smaller than $E_{\text{train}}$ meaning if $E_{\text{train}}$ is greater than required $E_{\text{new}}$ the problem needs to be reconsidered. Also the generalisation gap and $E_{\text{new}}$ decreases as the size of the training data increases, thus it could help a lot with reducing $E_{\text{new}}$. In addition it is possible to evaluate model complexity using $E_{\text{hold-out}}$, if $E_{\text{hold-out}} \approx E_{\text{train}}$, then underfitting is likely and it might be beneficial to increase complexity and if $E_{\text{train}}$ is close to zero while $E_{\text{hold-out}}$ is not, then overfitting is likely and it might be beneficial to decrease complexity instead. [p. 76-77]

- _What important aspect should be considered when selecting minibatches, according to the book?_

  When using minibatches it is vital to ensure that the different batches are balanced and represent the whole dataset. E.g. a training dataset with different output classes that are which is sorted by these output classes. Then the first minibatch might only include one output class and thus not give a good representation of the whole dataset. Therefore it is imprtand that the batches are formed randomly, for example by randomly shuffling the training data and then divide it into minibatches in an ordered manner. [p. 125]

- _Provide an example of modifications in a loss function and in data that can be done to take into account the data imbalance, according to the book._

  An example of modification in a loss function to take imbalance into account is to reflect that not predicting $y = 1$ correctly is a C times more costly than not predicting $y = -1$ (binary classification). The misclassification loss can be moddified to:

  $$
  L(y,\hat{y}) =
  \begin{cases}
  0 & \text{if } \hat{y} = y, \\
  1 & \text{if } \hat{y} \neq y \text{ and } y = -1, \\
  C & \text{if } \hat{y} \neq y \text{ and } y = 1.
  \end{cases}
  $$

  Other loss functions can be modified in a similar way. A similar effect can be achieved by e.g. duplicating al positive training data points C times in the training data instead of modifying the loss function. [p. 101-102]
